@@ -213,14 +213,9 @@ def compile_macros(macros):
     """
     Compile ROOT macros
     """
-    from quickstats.utils.root_utils import compile_macro
-    if macros is None:
-        macros = ["RooTwoSidedCBShape", "AsymptoticCLsTool", "RooFitObjects"]
-    else:
-        macros = macros.split(',')
-    for macro in macros:
-        compile_macro(macro)
-    
+    import quickstats
+    quickstats.compile_macros(macros)
+
 @click.command(name='harmonize_np')
 @click.argument('ws_files', nargs=-1)
 @click.option('-r', '--reference', required=True, help='Path to reference json file containing renaming scheme')
@@ -246,22 +241,29 @@ def harmonize_np(ws_files, reference, input_config_path, base_path, outfile):
 @click.option('-o', '--output_file', required=True, help='Name of the output workspace containing the '
                                                          'generated asimov dataset.')
 @click.option('-p', '--poi', required=True, help='Name of the parameter of interest (POI).')
-@click.option('--poi_val', type=float, required=True,
-              help='Generate asimov data with POI set at the specified value.')
+@click.option('--poi_val', type=float, default=None,
+              help='Generate asimov data with POI set at the specified value. '
+                   'If None, POI will be kept at the post-fit value if a fitting '
+                   'is performed or the pre-fit value if no fitting is performed.')
 @click.option('--poi_profile', type=float, default=None,
-              help='Perform NP profiling with POI set at the specified value.')
-@click.option('--globs_np_matching/--no_globs_np_matching', default=True, show_default=True,
+              help='Perform nuisance parameter profiling with POI set at the specified value. '
+                   'This option is only effective if do_fit is set to True. If None, POI is '
+                   'set floating (i.e. unconditional maximum likelihood estimate).')
+@click.option('--modify-globs/--keep-globs', default=True, show_default=True,
               help='Match the values of nuisance parameters and the corresponding global '
                    'observables when generating the asimov data. This is important for making '
                    'sure the asimov data has the (conditional) minimal NLL.')
-@click.option('--conditional_mle/--unconditional_mle', default=True, show_default=True,
-              help='Perform conditional maximum likelihood estimation. This option is effective '
-                   'only if poi_profile is specified. If True, best-fit values are obtained for '
-                   'mu = poi_profile. If False, the overal best-fit values are obtained.')
+@click.option('--do-fit/--no-fit', default=True, show_default=True,
+              help='Perform nuisance parameter profiling with a fit to the given dataset.')
 @click.option('--asimov_name', default="asimovData_{mu}", show_default=True,
               help='Name of the generated asimov dataset.')
 @click.option('-d', '--data', default='combData', show_default=True,
               help='Name of the dataset used in NP profiling.')
+@click.option('--constraint_option', default=0, show_default=True,
+              help='Customize the target of nuisance paramaters involved in the profiling.'
+                   'Case 0: All nuisance parameters are allowed to float;'
+                   'Case 1: Constrained nuisance parameters are fixed to 0. Unconstrained '
+                   'nuisrance parameters are allowed to float.')
 @click.option('-c', '--configuration', default=None,
               help='Path to the json configuration file containing'
                    ' the minimizer options for NP profiling.')
@@ -283,4 +285,37 @@ def generate_asimov(**kwargs):
     kwargs['poi_name'] = kwargs.pop("poi")
     task = AnalysisObject(filename=input_filename, data_name=data_name, **config)
     task.model.generate_asimov(**kwargs)
-    task.model.save(output_filename)
\ No newline at end of file
+    task.model.save(output_filename)
+
+
+@click.command(name='generate_standard_asimov')
+@click.option('-i', '--input_file', 'filename', required=True, 
+              help='Path to the input workspace file.')
+@click.option('-o', '--output_file', 'outname', required=True, 
+              help='Name of the output workspace containing the '
+                   'generated asimov dataset.')
+@click.option('-p', '--poi', 'poi_name', required=True, 
+              help='Name of the parameter of interest (POI).')
+@click.option('-s', '--poi_scale', type=float, default=1.0, show_default=True,
+              help='Scale factor applied to the poi value')
+@click.option('-t', '--asimov_types', default="0,1,2", show_default=True,
+              help='\b\nTypes of asimov dataset to generate separated by commas.\n'
+                   '\b 0: fit with POI fixed to 0\n'
+                   '\b 1: fit with POI fixed to 1\n'
+                   '\b 2: fit with POI free and set POI to 1 after fit\n'
+                   '\b 3: fit with POI and constrained NP fixed to 0\n'
+                   '\b 4: fit with POI fixed to 1 and constrained NP fixed to 0\n'
+                   '\b 5: fit with POI free and constrained NP fixed to 0 and set POI to 1 after fit\n'
+                   '\b -1: nominal NP with POI set to 0\n'
+                   '\b -2: nominal NP with POI set to 1\n')
+def generate_standard_asimov(**kwargs):
+    """
+    Generate standard Asimov dataset
+    """
+    from quickstats.components import AsimovGenerator
+    outname = kwargs.pop('outname')
+    asimov_types = kwargs.pop('asimov_types')
+    asimov_types = [int(t) for t in asimov_types.split(',')]
+    generator = AsimovGenerator(**kwargs)
+    generator.generate(asimov_types)
+    generator.save(outname)
\ No newline at end of file
