@@ -1,9 +1,9 @@
-#from quickstats.components.cli_parser import CLIParser
-#from quickstats.components.np_ranking_plotter import NPRankingPlotter
 from quickstats.components.extended_minimizer import ExtendedMinimizer
 from quickstats.components.extended_model import ExtendedModel
+from quickstats.components.analysis_object import AnalysisObject
+from quickstats.components.asimov_generator import AsimovGenerator
+from quickstats.components.asimov_generator import AsimovType
 from quickstats.components.nuisance_parameter_pull import NuisanceParameterPull
 from quickstats.components.roofit_plotter import RooFitPlotter
 from quickstats.components.nuisance_parameter_harmonizer import NuisanceParameterHarmonizer
-from quickstats.components.analysis_object import AnalysisObject
 from quickstats.components.asymptotic_cls import AsymptoticCLs
\ No newline at end of file
