@@ -11,7 +11,6 @@ from quickstats.utils import common_utils
 
 import ROOT
 
-from pdb import set_trace
 class AsymptoticCLs(AnalysisObject):
     
     _SIGMA_LABEL_ = {
@@ -449,7 +448,7 @@ class AsymptoticCLs(AnalysisObject):
         snapshot_0_name = self.nll_maps[asimov_0_nll]['snapshot']
         self.load_snapshot(snapshot_0_name)
         asimov_data_n = self.model.generate_asimov(self.poi.GetName(), poi_val=n_times_sigma,
-                                                   globs_np_matching=True)
+                                                   do_fit=False, modify_globs=True)
         asimov_n_nll = self.create_nll(asimov_data_n)
         self.load_conditional_snapshot(n_times_sigma, target="globs")
         self.load_conditional_snapshot(n_times_sigma, target="nuis")
@@ -479,12 +478,11 @@ class AsymptoticCLs(AnalysisObject):
         self.save_snapshot(self.nom_glob_name, target="globs")
         self.save_snapshot(self.nom_nuis_name, target="nuis")
         
-        # profile NP to mu = 0 if unblinded else no profiling
-        poi_profile = None if self.do_blind else 0
-        globs_np_matching = not self.do_blind
         asimov_data_0 = self.model.generate_asimov(self.poi.GetName(), poi_val=0,
-                                                   poi_profile=poi_profile, dataset=self.data,
-                                                   globs_np_matching=globs_np_matching)
+                                                   poi_profile=0, 
+                                                   do_fit=not self.do_blind,
+                                                   modify_globs=not self.do_blind,
+                                                   dataset=self.data)
         asimov_0_nll = self.create_nll(asimov_data_0)
         self.summary[0] = {'dataset': asimov_data_0, 'nll': asimov_0_nll}
         self.set_poi_value(0)
