@@ -28,8 +28,7 @@ class ExtendedModel(object):
         'conditional_nuis': 'conditionalNuis_{mu}',
         'nominal_globs': 'nominalGlobs',
         'nominal_nuis': 'nominalNuis',
-        'temp_globs': 'tmpGlobs',
-        'temp_nuis': 'tmpNuis',
+        'nominal_vars': 'nominalVars',
         'weight': 'weightVar',
         'dataset_args': 'obsAndWeight',
         'asimov': 'asimovData_{mu}',
@@ -112,6 +111,13 @@ class ExtendedModel(object):
                                                                             class_name,
                                                                             interp_code))
         return None
+    
+    @staticmethod
+    def remove_msg_topic(input_arguments=True, numeric_integration=True):
+        if input_arguments:
+            ROOT.RooMsgService.instance().getStream(1).removeTopic(ROOT.RooFit.InputArguments)
+        if numeric_integration:
+            ROOT.RooMsgService.instance().getStream(1).removeTopic(ROOT.RooFit.NumIntegration)
 
     @staticmethod
     def activate_binned_likelihood(ws):
@@ -217,8 +223,12 @@ class ExtendedModel(object):
         # load nuisance parameters
         nuisance_parameters = model_config.GetNuisanceParameters()
         if not nuisance_parameters:
-            raise RuntimeError('Failed to load nuisance parameters')
-        _PRINT_.info('INFO: Loaded nuisance parameters from model config')
+            #raise RuntimeError('Failed to load nuisance parameters')
+            _PRINT_.info("WARNING: No nuisance parameters found in the workspace. "
+                         "An empty set will be loaded by default.")
+            nuisance_parameters = ROOT.RooArgSet()
+        else:
+            _PRINT_.info('INFO: Loaded nuisance parameters from model config')
                 
         # Load global observables
         global_observables = model_config.GetGlobalObservables()
@@ -252,6 +262,8 @@ class ExtendedModel(object):
         if (self.snapshot_name) and self.workspace.getSnapshot(self.snapshot_name):
             self._workspace.loadSnapshot(self.snapshot_name)
             _PRINT_.info('INFO: Loaded snapshot "{}"'.format(self.snapshot_name))
+        
+        self.remove_msg_topic()
         return None
                 
     @staticmethod
@@ -511,7 +523,16 @@ class ExtendedModel(object):
             pdf_names  = [i.GetName() for i in pdf_list]
             size = len(nuis_names)
             return [(pdf_names[i], nuis_names[i], glob_names[i]) for i in range(size)]
-        return nuis_list, glob_list, pdf_list 
+        return nuis_list, glob_list, pdf_list
+    
+    def get_constrained_nuisance_parameters(self):
+        constrained_nuis, _, _ = self.pair_constraints()
+        return constrained_nuis
+    
+    def get_unconstrained_nuisance_parameters(self):
+        constrained_nuis = self.get_constrained_nuisance_parameters()
+        unconstrained_nuis = list(set(self.nuisance_parameters) - set(constrained_nuis))
+        return unconstrained_nuis    
     
     def get_all_constraints(self):
         all_constraints = ROOT.RooArgSet()
@@ -718,13 +739,17 @@ class ExtendedModel(object):
             raise RuntimeError("asimov data sum entries is nan")
         return asimov_data
 
-    def generate_asimov(self, poi_name:str, poi_val:float, poi_profile:Optional[float]=None,
-                        globs_np_matching:bool=False,
-                        conditional_mle:bool=True, do_import:bool=True,
+    def generate_asimov(self, poi_name:str, poi_val:Optional[float]=None, 
+                        poi_profile:Optional[float]=None,
+                        do_fit:bool=True,
+                        modify_globs:bool=True,
+                        do_import:bool=True,
                         asimov_name:Optional[str]=None,
                         channel_asimov_name:Optional[str]=None,
                         dataset:Optional["ROOT.RooDataSet"]=None,
-                        minimizer_options:Optional[Dict]=None, nll_options:Optional[Dict]=None,
+                        constraint_option:int=0,
+                        minimizer_options:Optional[Dict]=None, 
+                        nll_options:Optional[Union[Dict, List]]=None,
                         snapshot_names:Optional[Dict]=None):
         """
             Generate Asimov dataset.
@@ -741,23 +766,27 @@ class ExtendedModel(object):
             Arguments:
                 poi_name: str
                     Name of the parameter of interest (POI).
-                poi_val: float
-                    Generate asimov data with POI set at the specified value.
+                poi_val: (Optional) float
+                    Generate asimov data with POI set at the specified value. If None, POI will be kept
+                    at the post-fit value if a fitting is performed or the pre-fit value if no fitting
+                    is performed.
                 poi_profile: (Optional) float
-                    Perform nuisance parameter profiling. If None, nuisance parameters are kept at their
-                    initial values when generating the asimov data. Otherwise, a fit is performed by setting 
-                    the POI at the specified value (POI is fixed if conditional_mle=True, free otherwise) to
-                    obtain the best-fit nuisance parameter values before generating the asimov data.
-                    category label. Other keywords are: `{index}` which is the category index.
-                globs_np_matching: bool, default=False
+                    Perform nuisance parameter profiling with POI set at the specified value. This option
+                    is only effective if do_fit is set to True. If None, POI is set floating 
+                    (i.e. unconditional maximum likelihood estimate). 
+                do_fit: bool, default=True    
+                    Perform nuisance parameter profiling with a fit to the given dataset.
+                modify_globs: bool, default=True
                     Match the values of nuisance parameters and the corresponding global observables when
                     generating the asimov data. This is important for making sure the asimov data has the 
                     (conditional) minimal NLL.
+                constraint_option: int, default=0
+                    Customize the target of nuisance paramaters involved in the profiling.
+                    Case 0: All nuisance parameters are allowed to float;
+                    Case 1: Constrained nuisance parameters are fixed to 0. Unconstrained nuisrance
+                            parameters are allowed to float.
                 do_import: bool, default=True
-                    Import the generated asimov data to the current workspace.                    
-                conditional_mle: bool, default=True
-                    Perform conditional maximum likelihood estimation. If True, best-fit values are obtained
-                    for mu = poi_profile. If False, the overal best-fit values are obtained.
+                    Import the generated asimov data to the current workspace.
                 asimov_name: (Optional) str
                     Name of the generated asimov dataset. If None, defaults to "asimovData_{mu}" where
                     `{mu}` will be replaced by the value of `poi_val`. Other keywords are: `{mu_cond}`
@@ -771,17 +800,17 @@ class ExtendedModel(object):
                 minimizer_options: (Optional) dict
                     Options for minimization during nuisance parameter profiling. If None, defaults to
                     ExtendedMinimizer._DEFAULT_MINIMIZER_OPTION_
-                nll_options: (Optional) dict
-                    Options for NLl creation during nuisance parameter profiling. If None, defaults to
-                    ExtendedMinimizer._DEFAULT_NLL_OPTION_                    
+                nll_options: (Optional) dict, list
+                    Options for NLL creation during nuisance parameter profiling. If None, defaults to
+                    ExtendedMinimizer._DEFAULT_NLL_OPTION_
                 snapshot_names: (Optional) dict
                     A dictionary containing a map of the snapshot type and the snapshot names. The default
                     namings are stored in ExtendedModel._DEFAULT_NAMES_.
         """
         # define simplified ws variable names
         ws = self.workspace
-        globs = self.global_observables
-        nuis  = self.nuisance_parameters
+        all_globs = self.global_observables
+        all_nuis  = self.nuisance_parameters
         
         # define names used for various objects
         names = self._DEFAULT_NAMES_
@@ -791,31 +820,24 @@ class ExtendedModel(object):
             names['asimov'] = asimov_name
         if channel_asimov_name is not None:
             names['channel_asimov'] = channel_asimov_name
-        tmp_glob_name = names['temp_globs']
-        tmp_nuis_name = names['temp_nuis'] 
+        nom_vars_name = names['nominal_vars']
         nom_glob_name = names['nominal_globs']
         nom_nuis_name = names['nominal_nuis']
         con_glob_name = names['conditional_globs']
         con_nuis_name = names['conditional_nuis']
-        asimov_data_name = names['asimov'].format(mu=pretty_value(poi_val), 
-                                                  mu_cond=pretty_value(poi_profile))
-        channel_asimov_data_name = names['channel_asimov']
-        
-        poi = self.get_poi(poi_name)
-        if not nuis:
-            nuis = ROOT.RooArgSet()
-        nuis_list, glob_list = self.pair_nuis_and_glob_obs()
 
-        ws.saveSnapshot(tmp_glob_name, globs)
-        ws.saveSnapshot(tmp_nuis_name, nuis)
+        poi = self.get_poi(poi_name)
+        
+        # take snapshot of initial states of all variables
+        ws.saveSnapshot(nom_vars_name, self.workspace.allVars())
         if not ws.getSnapshot(nom_glob_name):
             _PRINT_.info('INFO: Saving snapshot "{}"'.format(nom_glob_name))
-            ws.saveSnapshot(nom_glob_name, globs)
+            ws.saveSnapshot(nom_glob_name, all_globs)
         if not ws.getSnapshot(nom_nuis_name):
             _PRINT_.info('INFO: Saving snapshot "{}"'.format(nom_nuis_name))
-            ws.saveSnapshot(nom_nuis_name, nuis)
-
-        if (poi_profile is not None):
+            ws.saveSnapshot(nom_nuis_name, all_nuis)
+        
+        if do_fit:
             from quickstats.components import ExtendedMinimizer
             if dataset is None:
                 dataset = self.data
@@ -825,31 +847,51 @@ class ExtendedModel(object):
             if nll_options is None:
                 nll_options = minimizer._DEFAULT_NLL_OPTION_
             minimizer.configure(**minimizer_options)
-            minimizer.configure_nll(self.nuisance_parameters, self.global_observables,
-                                    **nll_options)
-            poi.setVal(poi_profile)
-            if conditional_mle:
+            if constraint_option == 0:
+                pass
+            elif constraint_option == 1:
+                constrained_nuis = self.get_constrained_nuisance_parameters()
+                for nuis in constrained_nuis:
+                    nuis.setVal(0)
+                    nuis.setConstant(1)
+            else:
+                raise ValueError(f"unsupported constraint option: {constraint_option}")
+            if isinstance(nll_options, dict):
+                minimizer.configure_nll(all_nuis, all_globs, **nll_options)
+            elif isinstance(nll_options, list):
+                minimizer.nll_command_list = nll_options
+            else:
+                raise ValueError(f"unsupported nll options format")
+            if poi_profile is not None:
+                # conditional mle
+                poi.setVal(poi_profile)
                 poi.setConstant(1)
+            else:
+                # unconditional mle
+                poi.setConstant(0)
             minimizer.minimize()
-        poi.setVal(poi_val)
+
+        if poi_val is not None:
+            poi.setVal(poi_val)
         poi.setConstant(0)
-
-        for np, glob in zip(nuis_list, glob_list):
-            glob.setVal(np.getVal())
-
-        if (poi_profile is not None):
-            ws.saveSnapshot(con_glob_name.format(mu=poi_profile), globs)
-            ws.saveSnapshot(con_nuis_name.format(mu=poi_profile), nuis)
-        else:
-            ws.saveSnapshot(con_glob_name.format(mu=poi_val), globs)
-            ws.saveSnapshot(con_nuis_name.format(mu=poi_val), nuis)
-
-        if not globs_np_matching:
-            ws.loadSnapshot(nom_glob_name)
-            ws.loadSnapshot(nom_nuis_name)
-            
-        poi.setVal(poi_val)
+        poi_val = poi.getVal()
         
+        # match values of global observables to the corresponding NPs
+        if modify_globs:
+            constrained_nuis, constrained_globs = self.pair_nuis_and_glob_obs()
+            for np, glob in zip(constrained_nuis, constrained_globs):
+                glob.setVal(np.getVal())
+
+        if do_fit:
+            ws.saveSnapshot(con_glob_name.format(mu=poi_profile), all_globs)
+            ws.saveSnapshot(con_nuis_name.format(mu=poi_profile), all_nuis)
+        else:
+            ws.saveSnapshot(con_glob_name.format(mu=poi_val), all_globs)
+            ws.saveSnapshot(con_nuis_name.format(mu=poi_val), all_nuis)
+        
+        asimov_data_name = names['asimov'].format(mu=pretty_value(poi_val), 
+                                                  mu_cond=pretty_value(poi_profile))
+        channel_asimov_data_name = names['channel_asimov']
         sim_pdf = self.pdf
         if not isinstance(sim_pdf, ROOT.RooSimultaneous):
             asimov_data = self.generate_asimov_from_pdf(asimov_data_name, sim_pdf)
@@ -873,8 +915,10 @@ class ExtendedModel(object):
                                           ROOT.RooFit.WeightVar(weight_var))
         if do_import:
             getattr(ws, "import")(asimov_data)
-            
-        ws.loadSnapshot(nom_glob_name)
+            _PRINT_.info(f'INFO: Generated Asimov Dataset `{asimov_data_name}`')
+        
+        # load back a snapshot of all varaible's initial states
+        ws.loadSnapshot(nom_vars_name)
         
         return asimov_data
     
@@ -1097,7 +1141,10 @@ class ExtendedModel(object):
         if datasets is not None:
             for dataset in datasets:
                 if isinstance(dataset, str):
+                    dataset_name = dataset
                     dataset = self.workspace.data(dataset)
+                    if not dataset:
+                        raise ValueError(f"dataset `{dataset_name}` does not exist")
                 dataset_name = dataset.GetName()
                 distributions = self.get_dataset_distributions(dataset)
                 for category in distributions:
