@@ -22,9 +22,9 @@ def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax
                  workspace:Optional[str]=None, model_config:Optional[str]=None, data:str='combData', snapshot:str=None, 
                  profile:str="", fix:str="", hesse:bool=False, minos:bool=True, constrain:bool=True, 
                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, binned:bool=True,
-                 eps:float=1.0, strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, mpsplit:int=3, 
-                 verbose:int=0, max_calls:int=-1, max_iters:int=-1, optimize:int=2, offset:bool=False, 
-                 outname:str=None, cache:bool=False, detailed_output:bool=False):
+                 eps:float=1.0, strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, mpsplit:int=3,
+                 verbose:int=0, max_calls:int=-1, max_iters:int=-1, optimize:int=2, offset:bool=False,
+                 print_level:int=1, outname:str=None, cache:bool=False, detailed_output:bool=False):
     if cache:
         cached_result = load_cache(outname)
         if cached_result is not None:
@@ -75,7 +75,8 @@ def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax
         'max_calls'        : max_calls,
         'max_iters'        : max_iters,
         'hesse'            : hesse,
-        'verbose'          : verbose
+        'verbose'          : verbose,
+        'print_level'      : print_level
     }
     if minos:
         minimize_options['minos']     = True
@@ -120,7 +121,8 @@ def scan_nll(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:s
              profile:str="", fix:str="", hesse:bool=False, minos:bool=True, constrain:bool=True,
              minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, binned:bool=True,
              eps:float=1.0, strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, mpsplit:int=3, 
-             verbose:int=0, max_calls:int=-1, max_iters:int=-1, optimize:int=2, offset:bool=False, parallel:int=8):
+             verbose:int=0, print_level:int=1, max_calls:int=-1, max_iters:int=-1, optimize:int=2, 
+             offset:bool=False, parallel:int=8):
     start_time = time.time()
     points        = np.arange(poi_min, poi_max+poi_step, poi_step)
     unconditional = np.concatenate(([True], np.full((len(points)), False)))
@@ -139,24 +141,16 @@ def scan_nll(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:s
         os.makedirs(outdir)
     if not os.path.exists(cachedir):
         os.makedirs(cachedir)
-    if parallel:
-        arguments = (repeat(input_file), points, repeat(poi), repeat(vmin), repeat(vmax),
-                     unconditional, repeat(workspace),repeat(model_config), repeat(data),
-                     repeat(snapshot), repeat(profile), repeat(fix), repeat(hesse), repeat(minos),
-                     repeat(constrain), repeat(minimizer_type), repeat(minimizer_algo), repeat(num_cpu),
-                     repeat(binned), repeat(eps), repeat(strategy), repeat(fix_cache),
-                     repeat(fix_multi), repeat(mpsplit), repeat(verbose), repeat(max_calls), repeat(max_iters), 
-                     repeat(optimize), repeat(offset), outnames, repeat(cache)) 
-        max_workers = parallel if parallel != -1 else common_utils.get_cpu_count()
-        results = common_utils.parallel_run(evaluate_nll, *arguments, max_workers=max_workers)
-    else:
-        results = []
-        for point, uncond, outname_ in zip(points, unconditional, outnames):
-            nll = evaluate_nll(input_file, point, poi, vmin, vmax, uncond, workspace, model_config, data,
-                               snapshot, profile, fix, hesse, minos, constrain, minimizer_type, minimizer_algo,
-                               num_cpu, binned, eps, strategy, fix_cache, fix_multi, mpsplit, verbose, max_calls,
-                               max_iters, optimize, offset, outname_, cache)
-            results.append(nll)
+
+    arguments = (repeat(input_file), points, repeat(poi), repeat(vmin), repeat(vmax),
+                 unconditional, repeat(workspace),repeat(model_config), repeat(data),
+                 repeat(snapshot), repeat(profile), repeat(fix), repeat(hesse), repeat(minos),
+                 repeat(constrain), repeat(minimizer_type), repeat(minimizer_algo), repeat(num_cpu),
+                 repeat(binned), repeat(eps), repeat(strategy), repeat(fix_cache),
+                 repeat(fix_multi), repeat(mpsplit), repeat(verbose), repeat(max_calls), repeat(max_iters), 
+                 repeat(optimize), repeat(offset), repeat(print_level), outnames, repeat(cache))
+    results = common_utils.execute_multi_tasks(evaluate_nll, *arguments, parallel=parallel)
+
     uncond_nll = results[0]
     data = {"{{:.{}f}}".format(sd).format(poi_val):2*(nll-uncond_nll) \
             for poi_val, nll in zip(points[1:], results[1:])}
