@@ -1,3 +1,4 @@
+from typing import Optional, Union, Dict
 import os
 import sys
 import time
@@ -83,4 +84,17 @@ def json_load(fp, *args, **kwargs):
         data = json.load(fp, *args, **kwargs)
     except:
         raise RuntimeError(f"broken json input: {fp}")
-    return data
\ No newline at end of file
+    return data
+
+
+def parse_config(source:Optional[Union[Dict, str]]=None):
+    if source is None:
+        return {}
+    elif isinstance(source, str):
+        with open(source, 'r') as f:
+                config = json.load(f)
+        return config
+    elif isinstance(source, dict):
+        return source
+    else:
+        raise ValueError("invalid config input")
\ No newline at end of file
